/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>

#include <wintvchannel.h>

#if QT_VERSION < 200 // communication with windows manager
#include <kwm.h>
#else
#include <kwin.h>
#include <kmenubar.h>
#endif

#include <kiconloader.h>
#include <klocale.h>
#include <qmsgbox.h>
#include <qkeycode.h>
#include <qheader.h> 

#define BID_ADD     0
#define BID_EDIT    1
#define BID_DELETE  2
#define BID_SL      3
#define BID_SC      4
#define BID_MUP     5
#define BID_MDOWN   6 

#ifdef myDEBUG
#define chanDEBUG
#endif

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define PID_MUTE  0
#define PID_SPLIT 1
#define PID_INPUT 2

const char * mixIconNames[] = { "audio", "bass", "treble", "midi", "audio",
				"unknown", "ext", "microphone", "cd", "recmon",
				"audio", "recmon", "recmon", "recmon", "ext",
				"ext", "ext" };
#define mixIconNamesSize 17

#if QT_VERSION < 200
extern QPixmap BarIcon(const char* name);
#endif

winSlider::winSlider( int _devID, int _min, int _max, 
		      int _valr, int _vall, 
		      bool _stereo, bool _input, 
		      bool _recsrc, bool _muteable, 
		      QPixmap * pix, QWidget * parent, const char * name )
  : QWidget( parent, name ), devID(_devID), min(_min), max(_max), lval(_valr), 
    rval(_vall), stereo(_stereo), input(_input), recsrc(_recsrc), muteable(_muteable)
{
  sleft = new QSlider( this );
  sleft->setMinimumSize( 10, 10 );
  sleft->setMaximumSize( 32767, 32767 );
  sleft->setRange( min, max );
  sleft->setSteps( 1, 10 );
  sleft->setValue( max-lval );
  sleft->setTracking( TRUE );
  sleft->setTickmarks( QSlider::Left );
  sleft->setTickInterval( 10 );

  sright = new QSlider( this );
  sright->setMinimumSize( 10, 10 );
  sright->setMaximumSize( 32767, 32767 );
  sright->setRange( min, max );
  sright->setSteps( 1, 10 );
  sright->setValue( max-rval );
  sright->setTracking( TRUE );
  sright->setTickmarks( QSlider::Right );
  sright->setTickInterval( 10 );

  icon= 0;
  if ( pix ) {
    icon = new QLabel( this );
    icon->setGeometry(0,0,20,20);
    icon->setPixmap( *pix );
  }
  
  connect(sright,SIGNAL(valueChanged(int)),this,SLOT(valueRChangedI(int)));
  connect(sleft,SIGNAL(valueChanged(int)),this,SLOT(valueLChangedI(int)));

  sPopup=new QPopupMenu;
  if ( muteable ) sid[PID_MUTE]=sPopup->insertItem(i18n("&Mute"),PID_MUTE);
  if ( stereo ) sid[PID_SPLIT]=sPopup->insertItem(i18n("Un&split"),PID_SPLIT);
  if ( input ) sid[PID_INPUT]=sPopup->insertItem(i18n("Rec&Source"),PID_INPUT);
  connect(sPopup,SIGNAL(activated(int)),this,SLOT(popupSLOT(int)));

  sright->installEventFilter( this );
  sleft->installEventFilter( this );

  setMinimumSize( 15 , 60 );

  if ( !stereo ) {
    splited= false;
    sright->hide();
  } else
    splited= true;
  
  muted=false;
  mm=true;
}

winSlider::~winSlider ()
{
  delete sleft;
  delete sright;
}

void winSlider::popupSLOT( int id )
{
  switch (id) {
  case PID_MUTE:
    mute(!muted);
    break;
  case PID_SPLIT:
    split(!splited);
    break;
  case PID_INPUT:
    crecsrc(!recsrc);
    break;
  default:
    warning("winSlider: popup");
  }
}

void winSlider::setLValue( int v )
{
  lval=v;
  sleft->setValue( max-v );
  if ( !splited ) {
    sright->setValue( max-v );
  }
}

void winSlider::setRValue( int v )
{
  rval=v;
  sright->setValue( max-v );
}

void winSlider::valueLChangedI( int v )
{
  lval=max-v;
  if ( !splited ) {
    rval=max-v;
    sright->setValue( v );
  }
  emit valueChanged( devID, rval, lval );
}

void winSlider::valueRChangedI( int v )
{
  rval=max-v;
  emit valueChanged( devID, rval, lval );
}

void winSlider::split( bool sp )
{
  if ( !stereo ) return;
  if ( splited != sp ) {
    splited= sp;
    if ( !splited ) {
      sPopup->changeItem(i18n("&Split"),sid[PID_SPLIT]);
      //sleft->setTickmarks( QSlider::Both );
      sright->hide();
      int val= (lval+rval)/2;
      sleft->setValue( max-val);
      sright->setValue( max-val );
    } else {
      sPopup->changeItem(i18n("Un&split"),sid[PID_SPLIT]);
      //sleft->setTickmarks( QSlider::Left );
      sright->setValue( max-rval );
      sright->show();
    }
    resizeEvent(NULL);
    emit splitChanged( devID, sp );
  }
}

void winSlider::crecsrc( bool on )
{
  if ( !input ) return;
  recsrc= on;
  if ( recsrc ) {
    sleft->setBackgroundColor( red ); 
    sright->setBackgroundColor( red ); 
  } else {
    if (muted) {
      sleft->setBackgroundColor( black ); 
      sright->setBackgroundColor( black ); 
    } else {
      sleft->setBackgroundColor( colorGroup().mid() ); 
      sright->setBackgroundColor( colorGroup().mid() ); 
    }
  }
  emit recsrcChanged( devID, on );
}

void winSlider::mute( bool on )
{
  if ( !muteable ) return;
  muted= on;
  if ( muted ) {
    if ( !recsrc ) {
      sleft->setBackgroundColor( black ); 
      sright->setBackgroundColor( black ); 
    }
    sPopup->changeItem(i18n("&Unmute"),sid[PID_MUTE]);
  } else {
    if ( !recsrc ) {
      sleft->setBackgroundColor( colorGroup().mid() ); 
      sright->setBackgroundColor( colorGroup().mid() ); 
    }
    sPopup->changeItem(i18n("&Mute"),sid[PID_MUTE]);
  }
  emit muteChanged( devID, on );
}

void winSlider::setEnabled( bool on )
{
  mm=on;
  sleft->setEnabled(on);
  sright->setEnabled(on);
}

void winSlider::resizeEvent ( QResizeEvent *  )
{
  if ( icon ) {
    icon->setGeometry( width()/2 - 10, 0, 20, 20 );
    yoff= 25;
  } else {
    yoff= 0;
  }
  if (splited) {
    sleft->setGeometry( 0, yoff, width()/2-1, height()-yoff );
    sright->setGeometry( sleft->width(), yoff, width()/2-1, height()-yoff );
  } else {
    sleft->setGeometry( 0, yoff, width()-2, height()-yoff );
  }
}

#if QT_VERSION < 200
#define MOUSEBUTTONPRESSED Event_MouseButtonPress
#else
#define MOUSEBUTTONPRESSED QEvent::MouseButtonPress
#endif

bool winSlider::eventFilter( QObject *o, QEvent *e )
{
  // Lets see, if we have a "Right mouse button press"
  if (e->type() == MOUSEBUTTONPRESSED) {
    QMouseEvent *qme = (QMouseEvent*)e;
    if (qme->button() == RightButton) {
      sPopup->popup(QCursor::pos()/*mapToGlobal(qme->pos())*/);
      return true;
    }
  }
  return false;
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

int winMixer::glbId = 0; 

winMixer::winMixer( rcfile * _rc,
		    QWidget * parent, const char * name )
  : rc(_rc)
{
  myId=glbId++;

#if QT_VERSION < 200
  KIconLoader *loader = kapp->getIconLoader();
  QPixmap pm = loader->loadIcon("kwintv_48x48.png");
  setIcon(pm);
#else
  QPixmap pm = DesktopIcon("kwintv_48x48.png");
  setIcon(pm);
#endif

  config=GETAPPCONFIGINST;
  config->setGroup("CONFIG__mixer-"+QString().setNum(myId));

  // init sound mixer
  kmix= new mixer(myId);

  setIconText(i18n("kWinTV Mixer ")+QString(mixer::getMixerNameById(myId)));
  setCaption(i18n("kWinTV Mixer ")+QString(mixer::getMixerNameById(myId)));

  // set the kwintv mixing device
  // try line in, then other input devices
  if ( rc->mixerDevice == NO_MDEV ) {
    rc->mixerDevice= SOUND_MIXER_LINE;
    if ( ! kmix->isInputDevice(rc->mixerDevice) ) {
      int i;
      for ( i=0; i < SOUND_MIXER_NRDEVICES; i++ )
	if ( kmix->isInputDevice(i) ) {
	  rc->mixerDevice= i;
	  break;
	}
      if ( i == SOUND_MIXER_NRDEVICES ) {
	warning("winslider: was unable to find any sound mixer input device!");
	rc->mixerDevice= NO_MDEV;
      }
    }
  }

  // fetch all device names
  int devNr;
  char *devName;
  winSlider *ws;
  slidlist= new QIntDict<winSlider>;
  devName=kmix->getFirstDeviceName(&devNr);
  haveMixer=true;

  if ( !devName ) {
    haveMixer=false;
    warning("winmixer: no devices found!");
  } else while ( devName ) {
    if ( devNr == rc->mixerDevice ) {
      pm = BarIcon("kwintv_24x24.png");
    } else {
      if ( devNr < mixIconNamesSize ) {
	pm = BarIcon("mix_"+QString(mixIconNames[devNr])+".png");
      } else {
	pm = BarIcon("mix_unknown.png");
      }
    }

    ws= new winSlider(devNr,0,MIXER_MAX_VALUE,
		      kmix->getVolume(devNr,VOL_RIGHT),
		      kmix->getVolume(devNr,VOL_LEFT),
		      kmix->isStereoDevice(devNr),
		      kmix->isInputDevice(devNr),
		      kmix->isRecSource(devNr),true,&pm,this,
		      (const char *)QString(devName).lower());
    //ws->mute(true);
    ws->crecsrc(kmix->isRecSource(devNr));
    ws->split(config->readNumEntry("split_"+QString(devName).lower(),0));

    slidlist->insert(devNr,ws);

    if ( devNr == rc->mixerDevice ) {
      if ( !rc->use_master_volume ) {
#ifdef chanDEBUG
	debug("winmixer: set old volume: %d,%d",rc->volume_right,rc->volume_left);
#endif
//  	ws->setRValue(MIXER_MAX_VALUE-rc->volume_right);
//  	ws->setLValue(MIXER_MAX_VALUE-rc->volume_left);
	ws->setRValue(rc->volume_right);
	ws->setLValue(rc->volume_left);
	kmix->setStereoVolume(devNr,config->readNumEntry("vol_right",rc->volume_right),
			      config->readNumEntry("vol_left",rc->volume_left));
      } else { // use master vol
	rc->volume_right=kmix->getVolume(devNr,VOL_RIGHT);
	rc->volume_left=kmix->getVolume(devNr,VOL_LEFT);
#ifdef chanDEBUG
	debug("winmixer: set master volume: %d,%d",rc->volume_right,rc->volume_left);
#endif
      }
      QToolTip::add(ws,"kWinTV ("+QString(devName)+")");    
    } else {
      QToolTip::add(ws,devName);
    }

    connect(ws,SIGNAL(valueChanged(int,int,int)),this,SLOT(changeVol(int,int,int)));
    connect(ws,SIGNAL(muteChanged(int,bool)),this,SLOT(changeMute(int,bool)));
    connect(ws,SIGNAL(splitChanged(int,bool)),this,SLOT(changeSize(int,bool)));
    connect(ws,SIGNAL(recsrcChanged(int,bool)),this,SLOT(changeRecsrc(int,bool)));

    devName=kmix->getNextDeviceName(&devNr);
  }

  // setMinimumSize( *slidlist->count() , 20+10+100 );
  resize( 45*slidlist->count(), 20+10+100 );
  changeSize(0,false);

  if (rc->ol_restore) {
    QPoint pso(10,10);
    move(config->readPointEntry("xypos",&pso));
    //    resize(width(),height());
  }
}

winMixer::~winMixer()
{
#ifdef chanDEBUG
  debug("winmixer: ~winMixer()");
#endif
  if ( haveMixer ) {

    config->setGroup("CONFIG__mixer-"+QString().setNum(myId));
    config->writeEntry("xypos",pos());
    
    QIntDictIterator<winSlider> it( *slidlist );
    while (it.current()) {
#ifdef chanDEBUG_A
      debug("winmixer: save %s",it.current()->name());
#endif
      config->writeEntry("split_"+QString(it.current()->name())
			 ,it.current()->isSplited()?1:0);
      ++it;
    }
  }

  delete slidlist;
  delete kmix;

  //config->sync();
}

void winMixer::extVolSet( int rval, int lval )
{
  if ( !haveMixer || (rc->volume_right==rval && rc->volume_left == lval) ) return;
  rc->volume_right=rval;
  rc->volume_left=lval;
  (*slidlist)[rc->mixerDevice]->setRValue(rval);
  (*slidlist)[rc->mixerDevice]->setLValue(lval);
}
void winMixer::showMixer()
{
  if ( !haveMixer ) {
    QMessageBox::information( NULL, "kWinTV",
			      "No Sound Mixer aviable:"
			      "Error opening mixer device!",
			      "Ok",
			      0,      // Enter == button 0
			      0);
    if (isVisible()) hide();
    return;
  }
  isVisible()?hide():show();
}

void winMixer::changeVol( int devID, int rval, int lval )
{
  if ( !haveMixer ) return;
  kmix->setStereoVolume(devID,rval,lval);
  if ( devID == rc->mixerDevice ) {
    rc->volume_right=rval;
    rc->volume_left=lval;
  }
  emit volChanged(rval,lval);
}

void winMixer::changeRecsrc( int devID, bool on )
{
  kmix->setRecSource(devID,on);
}

void winMixer::extVolMute( bool on ) // on==true -> mute ( -> rc->sound_on=false )
{
#ifdef chanDEBUG
  debug("winMixer: extVolMute %d",on);
#endif
  if ( !haveMixer || (on == !rc->sound_on) ) return;
  rc->sound_on=!on;
  (*slidlist)[rc->mixerDevice]->mute(on);
}
void winMixer::extMuteRequest( bool on )
{
#ifdef chanDEBUG
  debug("winMixer: extMuteRequest %d",on);
#endif
  if ( !haveMixer || !rc->sound_on ) return;
  if ( on ) {
    kmix->setStereoVolume(rc->mixerDevice,0,0);
  } else {
    kmix->setStereoVolume(rc->mixerDevice,
			  (*slidlist)[rc->mixerDevice]->rVol(),
			  (*slidlist)[rc->mixerDevice]->lVol());
  }
}
void winMixer::changeMute( int devID, bool on ) // on==true -> mute ( -> rc->sound_on=false )
{
  if ( !haveMixer ) return;
  if ( on ) {
    kmix->setStereoVolume(devID,0,0);
  } else {
    kmix->setStereoVolume(devID,(*slidlist)[devID]->rVol(),(*slidlist)[devID]->lVol());
  }
  if ( devID == rc->mixerDevice ) {
    if ( on == !rc->sound_on ) return;
    rc->sound_on= !on;
    emit volMuteChanged( rc->sound_on );
  }
}

void winMixer::changeSize( int id, bool on )
{
#ifdef chanDEBUG
  debug("winMixer: changeSize");
#endif 
  xoff=5;
  QIntDictIterator<winSlider> it( *slidlist );
  while (it.current()) {
    xoff= 3 + xoff + (it.current()->isSplited()?46:24);
    ++it;
  }
  resize(xoff,height());
  setFixedSize(xoff,height());
}

void winMixer::resizeEvent ( QResizeEvent * )
{
  xoff=5;
  QIntDictIterator<winSlider> it( *slidlist );
  while (it.current()) {
    it.current()->setGeometry(xoff,5,it.current()->isSplited()?46:24,height()-10);
    xoff= 3 + xoff + (it.current()->isSplited()?46:24);
    ++it;
  }
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winProgramItem::winProgramItem( winTVChannelWidget::programNumberStyleEnum style,
				channel * _chan, 
				int _view, 
				QListView * parent=0, const char * name=0 )
  : QListViewItem(parent), pchannel(_chan), view(_view)
{
#ifdef chanDEBUG
  debug("winProgramItem: insert %d %s",pchannel->id,pchannel->name);
#endif
  QString qstr;

  switch (style) {
  case winTVChannelWidget::AmericanStyle: 
    {
      setText(0,pchannel->channelId);
    }
    break;
  case winTVChannelWidget::EuropeanStyle:
  case winTVChannelWidget::DefaultStyle:
  default: 
    {
      if ( pchannel->id < 10 ) 
	setText(0,qstr.sprintf(" %d",pchannel->id));
      else
	setText(0,qstr.setNum(pchannel->id));
    }
  }

  setText(1,pchannel->name);
}

void winProgramItem::setCText( int id, const char * text )
{
#ifdef chanDEBUG
  debug("winProgramItem: setCText id %d %s",id,text);
#endif

  setText(id,text);

  if ( id == 1 ) {
    if (pchannel->name != NULL) delete pchannel->name;
    pchannel->name = new char[strlen(text)+1];
    strcpy(pchannel->name,text);
  }

}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winTVChannelWidget::winTVChannelWidget(rcfile *_rc,
				       QWidget* parent, const char* name)
  : KTMainWindow( name ),rc(_rc)
{
#ifdef chanDEBUG
  debug("channel: begin of init");
#endif

#if QT_VERSION < 200
  KWM::setIcon(winId(), kapp->getIcon());
  KWM::setMiniIcon(winId(), kapp->getMiniIcon());

  KIconLoader *loader = kapp->getIconLoader();
  pm = loader->loadIcon("kwintv_logo.png");
  setIcon(pm);
#else
  QPixmap pm = DesktopIcon("kwintv_logo.png");
  setIcon(pm);
#endif

  setCaption((const char*)(QString(i18n("Program Browser"))));
  setIconText(i18n("Program Browser"));

  // toolbar
  toolbar = new KToolBar(this,"-",26);
  toolbar->setBarPos(KToolBar::Top);
  addToolBar(toolbar);

  ch_add_xpm     = BarIcon("filenew.png");
  ch_edit_xpm    = BarIcon("edit.png");
  ch_delete_xpm  = BarIcon("editcut.png");
  mup_xpm        = BarIcon("up.png");
  mdown_xpm      = BarIcon("down.png");
  cs_xpm         = BarIcon("delete.png");
  
  toolbar->insertButton(ch_add_xpm,BID_ADD,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("add new program"));
  toolbar->insertButton(ch_edit_xpm,BID_EDIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("edit this program"));
  toolbar->insertButton(ch_delete_xpm,BID_DELETE,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("cut this program"));
#if 1
  toolbar->insertSeparator();
  
  toolbar->insertButton(mup_xpm,BID_MUP,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("move up"));
  toolbar->insertButton(mdown_xpm,BID_MDOWN,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("move down"));
#endif
  toolbar->insertSeparator();

  toolbar->insertButton(cs_xpm,BID_SC,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("close program browser"));

  // program list
  programlist = new winProgramList( this );
  programlist->setGeometry( 5, 5, 140, 148 );
  programlist->setMinimumSize( 10, 10 );
  programlist->setMaximumSize( 32767, 32767 );

  programlist->setFrameStyle( 51 );
  programlist->setLineWidth( 2 );
  programlist->setMultiSelection ( false );
  programlist->setAllColumnsShowFocus( true );
  programlist->setTreeStepSize( 0 ); 
  programlist->setRootIsDecorated( false );

  programlist->addColumn(i18n("Nr."));
  programlist->setColumnWidth(0,30);
  programlist->addColumn(i18n("Program"));
  programlist->setColumnWidth(1,90+70);

  setView(programlist);
  programlist->setFocus();
  programNumberStyle = (programNumberStyleEnum)rc->ol_programnumberstyle;

  connect(programlist,SIGNAL(selectionChanged(QListViewItem *)),
	  this,SLOT(programSelectedSLOT(QListViewItem *)));
//    connect(programlist,SIGNAL(moveItemSIGNAL( int, int )),
//  	  this,SLOT(moveItemSLOT( int, int )));
  connect(programlist,SIGNAL(channelTextChangedSIGNAL(QListViewItem *)),
	  this,SLOT(programSelectedSLOT(QListViewItem *)));

#ifdef SHELLKEYS
  // timer for channel change by keyboard
  keyTimer = new QTimer(this);
  connect(keyTimer,SIGNAL(timeout()),this,SLOT(keyTimerSLOT()));
  programlist->setKeyTimeOut(rc->ol_keyTimeOut);
#endif

  // init window
  config=GETAPPCONFIGINST;
  
  setMinimumSize( 180,100 );

  if (rc->ol_restore) {
    config->setGroup("CONFIG__browser");
    QSize pso(210,260);
    resize(config->readSizeEntry("wsize",&pso));
    QPoint psp(0,0);
    move(config->readPointEntry("xypos",&psp));
  } else {
    resize( 210,260 );
  }
}

/* ------------------------------------------------------------------------------*/

winTVChannelWidget::~winTVChannelWidget()
{
  config->setGroup("CONFIG__browser");
  config->writeEntry("xypos",pos());
  config->writeEntry("wsize",size());
  config->sync();
  delete toolbar;
  delete programlist;
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::newChannelListSLOT( QList<channel> *clist )
{
  if ( clist->isEmpty() ) return;

  clistp= clist;

  int currentprog= clist->at();

  inserting=true;
  programlist->clear();
  int pos = 1;
  channel * cu= clist->first();
  while ( cu ) {
    cu->id = pos;
    (void) new winProgramItem( programNumberStyle, cu, 0xff, programlist );
    pos++;
    cu = clist->next();
  }
  
  inserting=false;

  clist->at(currentprog);

  programlist->setCurrentProgramItem( clist->at() );

#ifdef chanDEBUG
  debug("channel: channel list insertion done %d, %d",clist->count(),clist->at());
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::moveItemSLOT( int from, int to )
{
  if ( from == to || from == -1 || to == -1 || from == clistp->count() || to == clistp->count() )
    return;

#ifdef chanDEBUG
  debug("channel: move %d -> %d",from,to);
#endif

  inserting=TRUE;

  if ( from > to ) {
    channel *cfrom= new channel(*clistp->at(from));
    channel *cto= clistp->at(to);
    
    clistp->remove( from );
    clistp->insert( clistp->find(cto), cfrom );
  } else { // from < to 
    channel *cfrom= new channel(*clistp->at(from));
    channel *cto= clistp->at(to+1);
    
    clistp->remove( from );
    if ( cto == NULL ) 
      clistp->append(cfrom );
    else 
      clistp->insert( clistp->find(cto), cfrom );
  }
  inserting=FALSE;

  newChannelListSLOT( clistp );

  clistp->at(to);
  programlist->setCurrentProgramItem( clistp->at() );
  emit channelSelectedSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::activeChannelChangedSLOT()
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: activeChannelChangedSLOT");
#endif

  programlist->setCurrentProgramItem( clistp->at() );
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleVisibilitySLOT( QPoint *to )
{
  if (isVisible()) hide();
  else {
    if ( to ) {
#ifdef chanDEBUG
      debug("channel: setGeometry %d,%d,%d,%d",to->x(), to->y(), width(), height());
#endif
      setGeometry( to->x(), to->y(), width(), height() );
    }
    show();
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::programSelectedSLOT(QListViewItem * item)
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: programSelected %d",((winProgramItem*)item)->getChannelP()->id);
#endif

  if ( -1 != clistp->find(((winProgramItem*)item)->getChannelP()) )
    emit channelSelectedSIGNAL();

}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::channelEditEmitSLOT()
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: channelEditEmitSLOT");
#endif

  emit editChannelSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::buttonsSLOT(int button)
{
  programlist->hideLineEdit();

  switch (button) {
  case BID_SC:
    toggleVisibilitySLOT( NULL );
    break;
  case BID_ADD:
    emit addChannelSIGNAL();
    break;
  case BID_EDIT:
    emit editChannelSIGNAL();
    break;
  case BID_DELETE:
    emit deleteChannelSIGNAL();
    break;
  case BID_MUP:
    //    emit copyChannelSIGNAL();
    moveItemSLOT(clistp->at(),clistp->at()-1);
    break;
  case BID_MDOWN:
    //    emit pasteChannelSIGNAL();
    moveItemSLOT(clistp->at(),clistp->at()+1);
    break;
  default:
    // this should never happen...
    fatal("main: nonexisting button number");
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::setPopup(QPopupMenu *cp)
{
  programlist->setCPopup(cp);
}

/* ------------------------------------------------------------------------------*/

#ifdef SHELLKEYS
void winTVChannelWidget::keyPressEvent ( QKeyEvent *keyE )
{
#ifdef chanDEBUG
  debug("main: key pressed 0x%x",keyE->key());
#endif
  
  if ( (keyE->key() >= Key_0) && (keyE->key() <= Key_9) ) {
    if ( keyTimer->isActive() ) {
      keyTimer->stop();
      key2=keyE->key()-Key_0;
      keyTimerSLOT();
    } else {
      key1=keyE->key()-Key_0;
      key2=-1;
      keyTimer->start(rc->ol_keyTimeOut,true);
    }
  } else keyE->ignore();
}

void winTVChannelWidget::keyTimerSLOT()
{
  keyTimer->stop();

  int newitem = (-1==key2)?key1:key1*10+key2;

  QString id;
  id.sprintf("%2d",newitem);

  channel * it = clistp->first();
  while ( it->id != id ) {
    it= clistp->next();
  }

  if ( it != NULL ) {
    programlist->setCurrentProgramItem(newitem-1);
  }
}
#endif

// =============================================================================== 
// =============================================================================== 

winProgramList::winProgramList ( QWidget * parent=0, const char * name=0 ) 
  : QListView ( parent,name ), pressedButton( 0 ), oldY(0), 
    itemWasMoved( false ), cursorChanged( false )
{  
  returnpressed = false;

  keyTimeOut=1000;
  keyTimer = new QTimer(this);
  connect(keyTimer,SIGNAL(timeout()),this,SLOT(keyTimerSLOT()));
  lineedit = new QLineEdit(this);
  lineedit->hide();

#if QT_VERSION < 200
  connect(lineedit,SIGNAL(textChanged(const char *)),this,SLOT(textChangedSLOT(const char *)));
#else
  connect(lineedit,SIGNAL(textChanged(const QString&)),this,SLOT(textChangedSLOT(const QString&)));
#endif

  connect(lineedit,SIGNAL(returnPressed()),this,SLOT(returnPressedSLOT()));

  connect(this,SIGNAL( doubleClicked ( QListViewItem * ) ),this,SLOT( doubleClickedSLOT( QListViewItem * ) ));
  connect(this,SIGNAL( returnPressed ( QListViewItem * ) ),this,SLOT( doubleClickedSLOT( QListViewItem * ) ));
  connect(this,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(selectionChangedSLOT( QListViewItem * ) ));
}

winProgramList::~winProgramList()
{
  delete keyTimer;
  delete lineedit;
}

bool winProgramList::setCurrentProgramItem( int item )
{
  int pos = 0;
  QListViewItem *nitem = firstChild();
  while ( nitem != NULL && pos < item ) {
    pos++;
    nitem = nitem->nextSibling();
  }
  if ( nitem != NULL ) {
    setCurrentItem(nitem);
    setSelected(nitem,true);
    ensureItemVisible(nitem);
    return true;
  }

  return false;
}

void winProgramList::doubleClickedSLOT( QListViewItem *item )
{
#ifdef chanDEBUG
  debug("winProgramList: doubleClicked");
#endif
  if ( item == NULL || returnpressed ) {
    returnpressed = false;
    return;
  }
  eitem=(winProgramItem*)item;
  QRect rct( itemRect( item ) );
  setSelected(item,false);
  lineedit->setGeometry(rct.x()+columnWidth(0),rct.y()+header()->height(),columnWidth(1),rct.height()+2);
  lineedit->setText(item->text(1));
  lineedit->show();
  lineedit->raise();
  lineedit->setFocus();
}

void winProgramList::selectionChangedSLOT( QListViewItem * )
{
  if ( lineedit->isVisible() ) {
    lineedit->hide();
    emit channelTextChangedSIGNAL(eitem);
  }
}

//#if QT_VERSION < 200
void winProgramList::textChangedSLOT( const char * text)
{
  if (eitem!=NULL) eitem->setCText(1,text);
}
//#else
void winProgramList::textChangedSLOT( const QString& text)
{
  if (eitem!=NULL) eitem->setCText(1,text);
}
//#endif

void winProgramList::returnPressedSLOT()
{
#ifdef chanDEBUG
  debug("winProgramList: LE return pressed");
#endif
  if ( lineedit->isVisible() ) {
    returnpressed = true;
    lineedit->hide();
    setCurrentItem(eitem);
    setSelected(eitem,true);
    emit channelTextChangedSIGNAL(eitem);
  }
}

void winProgramList::leaveEvent ( QEvent * )
{
  if ( lineedit->isVisible() ) {
    lineedit->hide();
    setCurrentItem(eitem);
    setSelected(eitem,true);
    emit channelTextChangedSIGNAL(eitem);
  }
}

void winProgramList::hideLineEdit()
{
  if ( lineedit->isVisible() ) {
    lineedit->hide();
    setCurrentItem(eitem);
    setSelected(eitem,true);
    //emit channelTextChangedSIGNAL(eitem);
  }
}

void winProgramList::keyPressEvent ( QKeyEvent *keyE )
{
#ifdef chanDEBUG
  debug("winProgramList: key pressed 0x%x",keyE->key());
#endif
  
  if ( (keyE->key() >= Key_0) && (keyE->key() <= Key_9) ) {
    if ( keyTimer->isActive() ) {
      keyTimer->stop();
      key2=keyE->key()-Key_0;
      keyTimerSLOT();
    } else {
      key1=keyE->key()-Key_0;
      key2=-1;
      keyTimer->start(keyTimeOut,true);
    }
  } else {
    keyE->ignore();
    QListView::keyPressEvent( keyE );
  }
}

void winProgramList::keyTimerSLOT()
{
  int newitem = (-1==key2)?key1:key1*10+key2;

  keyTimer->stop();

  QString id;
  id.sprintf("%2d",newitem);

  QListViewItem *nitem = firstChild();
  while ( nitem != NULL && nitem->text(0) != id ) {
    nitem = nitem->nextSibling();
  }

  if ( nitem != NULL ) {
    setCurrentItem(nitem);
    setSelected(nitem,true);
    ensureItemVisible(nitem);
    //repaintItem(nitem);
  }
}

#if QT_VERSION >= 200
#define RightButton Qt::RightButton
#endif

void winProgramList::mouseReleaseEvent ( QMouseEvent * me )
{
#ifdef chanDEBUG
  debug("winProgramList: mouseReleaseEvent");
#endif

  if (me->button() == RightButton) {
    hideLineEdit();
    cPopup->popup(QCursor::pos());
    return;
  }

#ifdef DRAGDROP
  if ( !pressedButton ) QListView::mouseReleaseEvent( me );

  switch (me->button()) {
  case LeftButton:
#ifdef chanDEBUG
    debug("channelListBox: mouseReleaseEvent, LeftButton released");
#endif
    if ( itemWasMoved ) {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item moved, LeftButton released");
#endif
      QListViewItem *targetItem= currentItem();
      if ( targetItem && (toMoveItem != targetItem) ) {
#ifdef chanDEBUG
	debug("channelListBox: mouseReleaseEvent, moving item to %s",(const char *)targetItem->text(1));
#endif
	emit moveItemSIGNAL( toMoveItem, targetItem );
      } else {
	setCurrentItem( currentItem() );
      }
      itemWasMoved = false;
      pressedButton= 0;
    } else {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item not moved, LeftButton released");
#endif
      itemWasMoved = false;
      pressedButton= 0;
    }
  } // switch
#endif
}

// =============================================================================== 
#ifdef DRAGDROP
void winProgramList::mousePressEvent ( QMouseEvent * me )
{
  QListView::mousePressEvent ( me );
  
  if (me->button() == LeftButton) {
#ifdef chanDEBUG
    debug("winProgramList: mousePressEvent, LeftButton");
#endif
    pressedButton = LeftButton;
    toMoveItem= itemAt( me->pos() );
  } else 
    pressedButton = 0;
  
}
#endif
// =============================================================================== 
#ifdef DRAGDROP
void winProgramList::mouseMoveEvent ( QMouseEvent * me )
{
  if ( !pressedButton ) QListView::mouseMoveEvent( me );

  if (pressedButton == LeftButton) {

#ifdef chanDEBUG
    debug("winProgramList: mouseMoveEvent, LeftButton");
#endif

    oldY= me->y();

    if ( geometry().contains( me->pos() ) && (itemAt( me->pos() ) != NULL) ) {
      lastVisItem=itemAt( me->pos() );
      setCurrentItem( lastVisItem );
      /*
      ensureItemVisible( lastVisItem );

      if ( itemAt( me->pos() ) != lastVisItem ) {
	QPoint pos( mapToGlobal(me->pos()).x() , itemPos( lastVisItem ));
	QCursor::setPos( mapFromGlobal(pos) );
      }
      */
      itemWasMoved=true;
      if ( !cursorChanged ) {
	// change mouse cursor
	cursorChanged= true;
      }
#ifdef chanDEBUG
      //debug("winProgramList: mouseMoveEvent: move");
#endif
    } else {
#ifdef chanDEBUG
      debug("winProgramList: mouseMoveEvent: exit");
#endif
      itemWasMoved=false;
      setCurrentItem( toMoveItem );
      //ensureItemVisible ( currentItem() );
      pressedButton=0;
      cursorChanged= false;
    }
  }
}
#endif
// =============================================================================== 
#if 0
void winProgramList::resizeEvent( QResizeEvent *  )
{
  //setGeometry( 2, 2, width()-4, height()-4 );
}
#endif
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvchannel.moc"

// local Variables: ***
// compile-command-1: "make kwintvg" ***
// compile-command-2: "make" ***
// End: ***
